%% Copyright (C) Naiyan Wang and Dit-Yan Yeung.
%% Learning A Deep Compact Image Representation for Visual Tracking. (NIPS2013')
%% All rights reserved.

% initialize variables
% clc; clear;
function results=run_CNNT(seq, res_path, bSaveImage)
    addpath('affineUtility');
    addpath('drawUtility');
    addpath('imageUtility');
    %addpath('../../caffe_old/matlab/caffe');
    addpath('/ghome/sliay/documents/caffe_old/matlab/caffe');
    rng('default');
    if isfield(seq, 'opt')
        opt = seq.opt;
    else
        trackparam_CNNT;
    end
    rect=seq.init_rect;
    p = [rect(1)+rect(3)/2, rect(2)+rect(4)/2, rect(3), rect(4)];
    frame = imread(seq.s_frames{1});
    if size(frame,3)==1
        frame = repmat(frame,[1,1,3]);
    end
    
    scaleHeight = size(frame, 1) / opt.normalHeight;
    scaleWidth = size(frame, 2) / opt.normalWidth;
    p(1) = p(1) / scaleWidth;
    p(3) = p(3) / scaleWidth;
    p(2) = p(2) / scaleHeight;
    p(4) = p(4) / scaleHeight;
    frame = imresize(frame, [opt.normalHeight, opt.normalWidth]);
    frame = double(frame);
    posNum = 8;
    fixedNegNum = 32;
    
   
    reportRes = [];
    
    % Sample 8 negative templates for initialization
    tmpl.basis(:, posNum + 1 : posNum + fixedNegNum) = sampleNeg(frame, p, 100, 0);

%     % draw initial track window
%    drawopt = easydraw([], 0, frame, p);
    
    finalParam.est = p;
    finalParam.lastUpdate = 1;
    % init CNNT using pretrain data and first frame data
    caffe('set_net_id', 0);
    caffe('init_finetune', '/ghome/sliay/documents/caffe_old/examples/objectness/imagenet_deploy_solver.prototxt', '/ghome/sliay/documents/caffe_objectness_train_iter_100000'); 
    for i = 1 : 4
        % Sample 8 positive templates for initialization
        [tmpl.basis(:, 1:posNum), tmpl.cord(1:posNum, :)] = samplePos_CNNT(frame, p, 100, 1,4);
        initCNNT(tmpl, posNum + fixedNegNum, fixedNegNum, 5);
    end
    caffe('set_net_id', 1);
    caffe('init_finetune', '/ghome/sliay/documents/caffe_old/examples/objectness/imagenet_deploy_solver.prototxt', '/ghome/sliay/documents/caffe_objectness_train_iter_100000'); 
    for i = 1 : 4
        % Sample 8 positive templates for initialization
        [tmpl.basis(:, 1:posNum), tmpl.cord(1:posNum, :)] = samplePos_CNNT(frame, p, 100, 1,4);
        initCNNT(tmpl, posNum + fixedNegNum, fixedNegNum, 5);
    end
    fixedPos.basis(:, 1:posNum) = tmpl.basis(:, 1:posNum);
    fixedPos.cord(1:posNum, :) = tmpl.cord( 1:posNum, :);
    % track the sequence from frame 1 onward
    duration = 0; tic;
    for f = 1:size(seq.s_frames,1)  
      frame = imread(seq.s_frames{f});
      if size(frame,3)==1
        frame = repmat(frame, [1,1,3]);
      end  
      frame = imresize(frame, [opt.normalHeight, opt.normalWidth]);
      frame = double(frame);
      numscale = 4;
%       if param.lastUpdate == 0
%         numscale = 4;
%         disp('======not found======');
%       end
      [tmpl.basis] = samplePos_CNNT(frame, finalParam.est, 100, 0, numscale);
      
%       try
      tmpl.basis(:, numscale + 1 : numscale + fixedNegNum) = sampleNeg(frame, finalParam.est, 100, 0);
%       catch
%           disp('aa');
%       end
      % do tracking
      caffe('set_net_id', 0);
      fixedParam = estwarp_condens_CNNT(frame, finalParam, tmpl, f, 100);
      caffe('set_net_id', 1);
      updateParam = estwarp_condens_CNNT(frame, finalParam, tmpl, f, 100);
      if fixedParam.conf > updateParam.conf
          finalParam = fixedParam;
          disp('using net 0');
      else
          finalParam = updateParam;
          disp('using net 1');
      end
      fprintf('fixedParam conf %f\nparam conf \t%f\t %d\n', fixedParam.conf, updateParam.conf, f);
      
      %% update net 0
      if fixedParam.conf > 0.8 && size(fixedParam.negSample, 2) > 0
        caffe('set_net_id', 0);
        fprintf('updating net 0\n');
        clear tmpl;
        [tmpl.basis, tmpl.cord] = samplePos_CNNT(frame, fixedParam.est, 100, 1, 4);
        tmpl.basis = [tmpl.basis fixedPos.basis];
        tmpl.cord = [tmpl.cord; fixedPos.cord];
        start = posNum * 2;
%         start = posNum;
        negNum = size(fixedParam.negSample, 2);
        tmpl.basis(:, start + 1 : start + negNum) = fixedParam.negSample;
        initCNNT(tmpl, start + negNum, negNum, 1);
      end

      p = finalParam.est;
      %% update net 1
      if  size(updateParam.negSample, 2) > 0
          caffe('set_net_id', 1);
          fprintf('updating net 1\n');
          clear tmpl;
          if finalParam.conf > 1e-4
              [tmpl.basis, tmpl.cord] = samplePos_CNNT(frame, p, 100, 1, 4);
              tmpl.basis = [tmpl.basis fixedPos.basis];
              tmpl.cord = [tmpl.cord; fixedPos.cord];
              start = posNum * 2;
%                 start = posNum;
          else 
              start = 0;
          end
          negNum = size(updateParam.negSample, 2);
          tmpl.basis(:, start + 1 : start + negNum) = updateParam.negSample;
          initCNNT(tmpl, start + negNum, negNum, 1);
      end

%      drawopt = easydraw(drawopt, f, frame, p);
      drawnow;
      p(1) = p(1) * scaleWidth;
      p(3) = p(3) * scaleWidth;
      p(2) = p(2) * scaleHeight;
      p(4) = p(4) * scaleHeight;
      rect = [p(1) - p(3) / 2, p(2) - p(4) / 2, p(3), p(4)];
      reportRes = [reportRes; round(rect)];
      % save('man1.mat','reportRes');
    end
    duration = duration + toc;
    fprintf('%d frames took %.3f seconds : %.3fps\n',f,duration,f/duration);
    results.res=reportRes;
    results.type='rect';
    results.tmplsize = opt.tmplsize;
    results.fps = f/duration;
end
