function sodlt_vot
% ncc VOT integration example
% 
% This function is an example of tracker integration into the toolkit.
% The implemented tracker is a very simple NCC tracker that is also used as
% the baseline tracker for challenge entries.
%

%% modify this path to include the tracker directory
addpath(genpath('/ghome/sliay/documents/CNNTracker'));
%% modifly this path to include the caffe directory
addpath('/ghome/sliay/documents/caffe/matlab/caffe');
warning off;


% *************************************************************
% VOT: Always call exit command at the end to terminate Matlab!
% *************************************************************
cleanup = onCleanup(@() exit() );

% *************************************************************
% VOT: Set random seed to a different value every time.
% *************************************************************
RandStream.setGlobalStream(RandStream('mt19937ar', 'Seed', sum(clock)));
%rng('default');

% **********************************
% VOT: Get initialization data
% **********************************
[handle, I, region] = vot('rectangle');
im = imread(I);

opt.maxbasis = 10;
opt.updateThres = 0.8;
opt.condenssig = 0.01;
opt.tmplsize = [32, 32];
opt.normalWidth = 1280;
opt.normalHeight = 720;

p = [region(1)+region(3)/2, region(2)+region(4)/2, region(3), region(4)];

frame = im;
if size(frame,3)==1
    frame = repmat(frame, [1,1,3]);
end  
scaleHeight = size(frame, 1) / opt.normalHeight;
scaleWidth = size(frame, 2) / opt.normalWidth;
frame = imresize(frame, [opt.normalHeight, opt.normalWidth]);
frame = double(frame);

p(1) = p(1) / scaleWidth;
p(3) = p(3) / scaleWidth;
p(2) = p(2) / scaleHeight;
p(4) = p(4) / scaleHeight;

posNum = 8;
fixedNegNum = 32;

tmpl.basis(:, posNum + 1 : posNum + fixedNegNum) = sampleNeg(frame, p, 100, 0);

finalParam.est = p;
finalParam.lastUpdate = 1;
% init CNNT using pretrain data and first frame data
caffe('set_net_id', 0);
caffe('init_finetune', '/ghome/sliay/documents/caffe_old/examples/objectness/imagenet_deploy_solver.prototxt', '/ghome/sliay/documents/caffe_objectness_train_iter_100000'); 
for i = 1 : 4
    % Sample 8 positive templates for initialization
    [tmpl.basis(:, 1:posNum), tmpl.cord(1:posNum, :)] = samplePos_CNNT(frame, p, 100, 1,4);
    initCNNT(tmpl, posNum + fixedNegNum, fixedNegNum, 5);
end
caffe('set_net_id', 1);
caffe('init_finetune', '/ghome/sliay/documents/caffe_old/examples/objectness/imagenet_deploy_solver.prototxt', '/ghome/sliay/documents/caffe_objectness_train_iter_100000'); 
for i = 1 : 4
    % Sample 8 positive templates for initialization
    [tmpl.basis(:, 1:posNum), tmpl.cord(1:posNum, :)] = samplePos_CNNT(frame, p, 100, 1,4);
    initCNNT(tmpl, posNum + fixedNegNum, fixedNegNum, 5);
end
fixedPos.basis(:, 1:posNum) = tmpl.basis(:, 1:posNum);
fixedPos.cord(1:posNum, :) = tmpl.cord( 1:posNum, :);

% detector = cv.FeatureDetector('SURF');
% extractor = cv.DescriptorExtractor('SURF');
% matcher = cv.DescriptorMatcher('FlannBased');
% last_keypoints = detector.detect(im);
% last_descriptors = extractor.compute(im, last_keypoints);
f = 1;
while true
    numscale = 4;
    [tmpl.basis] = samplePos_CNNT(frame, finalParam.est, 100, 0, numscale);
      
    tmpl.basis(:, numscale + 1 : numscale + fixedNegNum) = sampleNeg(frame, finalParam.est, 100, 0);
    

    caffe('set_net_id', 0);
    fixedParam = estwarp_condens_CNNT(frame, finalParam, tmpl, f, 100);
    caffe('set_net_id', 1);
    updateParam = estwarp_condens_CNNT(frame, finalParam, tmpl, f, 100);
    if fixedParam.conf > updateParam.conf
      finalParam = fixedParam;
      disp('using net 0');
    else
      finalParam = updateParam;
      disp('using net 1');
    end
    fprintf('fixedParam conf %f\nparam conf \t%f\t %d\n', fixedParam.conf, updateParam.conf, f);

    %% update net 0
    if fixedParam.conf > 0.8 && size(fixedParam.negSample, 2) > 0
        caffe('set_net_id', 0);
        fprintf('updating net 0\n');
        clear tmpl;
        [tmpl.basis, tmpl.cord] = samplePos_CNNT(frame, fixedParam.est, 100, 1, 4);
        tmpl.basis = [tmpl.basis fixedPos.basis];
        tmpl.cord = [tmpl.cord; fixedPos.cord];
        start = posNum * 2;
        %         start = posNum;
        negNum = size(fixedParam.negSample, 2);
        tmpl.basis(:, start + 1 : start + negNum) = fixedParam.negSample;
        initCNNT(tmpl, start + negNum, negNum, 1);
    end

    p = finalParam.est;
    %% update net 1
    if  size(updateParam.negSample, 2) > 0
      caffe('set_net_id', 1);
      fprintf('updating net 1\n');
      clear tmpl;
      if finalParam.conf > 1e-4
          [tmpl.basis, tmpl.cord] = samplePos_CNNT(frame, p, 100, 1, 4);
          tmpl.basis = [tmpl.basis fixedPos.basis];
          tmpl.cord = [tmpl.cord; fixedPos.cord];
          start = posNum * 2;
    %                 start = posNum;
      else 
          start = 0;
      end
      negNum = size(updateParam.negSample, 2);
      tmpl.basis(:, start + 1 : start + negNum) = updateParam.negSample;
      initCNNT(tmpl, start + negNum, negNum, 1);
    end


    p(1) = p(1) * scaleWidth;
    p(3) = p(3) * scaleWidth;
    p(2) = p(2) * scaleHeight;
    p(4) = p(4) * scaleHeight;

    rect = [p(1) - p(3) / 2, p(2) - p(4) / 2, p(3), p(4)];

    % **********************************
    % VOT: Report position for frame
    % **********************************
    if f ~= 1
        region = round(rect);
        handle = handle.report(handle, region);
    end
    
    % **********************************
    % VOT: Get next frame
    % **********************************
    [handle, I] = handle.frame(handle);
    
    if isempty(I)
        break;
    end;
    im = imread(I);
    frame = im;
    if size(frame,3)==1
        frame = repmat(frame, [1,1,3]);
    end  
    frame = imresize(frame, [opt.normalHeight, opt.normalWidth]);
    frame = double(frame);
    f = f + 1;
end

% **********************************
% VOT: Output the results
% **********************************
handle.quit(handle);

end
